# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging as std_logging
from collections import OrderedDict
import re
from typing import Dict, Callable, Mapping, MutableMapping, MutableSequence, Optional, Sequence, Tuple, Type, Union

from googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1 import gapic_version as package_version

from google.api_core.client_options import ClientOptions
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.auth import credentials as ga_credentials   # type: ignore
from google.oauth2 import service_account              # type: ignore
import cloudsdk.google.protobuf


try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

from google.api_core import operation  # type: ignore
from google.api_core import operation_async  # type: ignore
from cloudsdk.google.protobuf import empty_pb2  # type: ignore
from cloudsdk.google.protobuf import field_mask_pb2  # type: ignore
from cloudsdk.google.protobuf import timestamp_pb2  # type: ignore
from googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.services.semantic_governance_policy_service import pagers
from googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types import semantic_governance_policy_service
from .transports.base import SemanticGovernancePolicyServiceTransport, DEFAULT_CLIENT_INFO
from .transports.grpc_asyncio import SemanticGovernancePolicyServiceGrpcAsyncIOTransport
from .client import SemanticGovernancePolicyServiceClient

try:
    from google.api_core import client_logging  # type: ignore
    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)

class SemanticGovernancePolicyServiceAsyncClient:
    """Manages SemanticGovernancePolicies.
    A SemanticGovernancePolicy is a resource that represents a
    user-specified policy applied to an Agent and optionally a
    specific Tool within that Agent.
    """

    _client: SemanticGovernancePolicyServiceClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = SemanticGovernancePolicyServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = SemanticGovernancePolicyServiceClient.DEFAULT_MTLS_ENDPOINT
    _DEFAULT_ENDPOINT_TEMPLATE = SemanticGovernancePolicyServiceClient._DEFAULT_ENDPOINT_TEMPLATE
    _DEFAULT_UNIVERSE = SemanticGovernancePolicyServiceClient._DEFAULT_UNIVERSE

    semantic_governance_policy_path = staticmethod(SemanticGovernancePolicyServiceClient.semantic_governance_policy_path)
    parse_semantic_governance_policy_path = staticmethod(SemanticGovernancePolicyServiceClient.parse_semantic_governance_policy_path)
    common_billing_account_path = staticmethod(SemanticGovernancePolicyServiceClient.common_billing_account_path)
    parse_common_billing_account_path = staticmethod(SemanticGovernancePolicyServiceClient.parse_common_billing_account_path)
    common_folder_path = staticmethod(SemanticGovernancePolicyServiceClient.common_folder_path)
    parse_common_folder_path = staticmethod(SemanticGovernancePolicyServiceClient.parse_common_folder_path)
    common_organization_path = staticmethod(SemanticGovernancePolicyServiceClient.common_organization_path)
    parse_common_organization_path = staticmethod(SemanticGovernancePolicyServiceClient.parse_common_organization_path)
    common_project_path = staticmethod(SemanticGovernancePolicyServiceClient.common_project_path)
    parse_common_project_path = staticmethod(SemanticGovernancePolicyServiceClient.parse_common_project_path)
    common_location_path = staticmethod(SemanticGovernancePolicyServiceClient.common_location_path)
    parse_common_location_path = staticmethod(SemanticGovernancePolicyServiceClient.parse_common_location_path)

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SemanticGovernancePolicyServiceAsyncClient: The constructed client.
        """
        return SemanticGovernancePolicyServiceClient.from_service_account_info.__func__(SemanticGovernancePolicyServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SemanticGovernancePolicyServiceAsyncClient: The constructed client.
        """
        return SemanticGovernancePolicyServiceClient.from_service_account_file.__func__(SemanticGovernancePolicyServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(cls, client_options: Optional[ClientOptions] = None):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return SemanticGovernancePolicyServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> SemanticGovernancePolicyServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            SemanticGovernancePolicyServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = SemanticGovernancePolicyServiceClient.get_transport_class

    def __init__(self, *,
            credentials: Optional[ga_credentials.Credentials] = None,
            transport: Optional[Union[str, SemanticGovernancePolicyServiceTransport, Callable[..., SemanticGovernancePolicyServiceTransport]]] = "grpc_asyncio",
            client_options: Optional[ClientOptions] = None,
            client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
            ) -> None:
        """Instantiates the semantic governance policy service async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,SemanticGovernancePolicyServiceTransport,Callable[..., SemanticGovernancePolicyServiceTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport to use.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the SemanticGovernancePolicyServiceTransport constructor.
                If set to None, a transport is chosen automatically.
                NOTE: "rest" transport functionality is currently in a
                beta state (preview). We welcome your feedback via an
                issue in this library's source repository.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = SemanticGovernancePolicyServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,

        )

        if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(std_logging.DEBUG):  # pragma: NO COVER
            _LOGGER.debug(
                "Created client `google.cloud.aiplatform_v1.SemanticGovernancePolicyServiceAsyncClient`.",
                extra = {
                    "serviceName": "google.cloud.aiplatform.v1.SemanticGovernancePolicyService",
                    "universeDomain": getattr(self._client._transport._credentials, "universe_domain", ""),
                    "credentialsType": f"{type(self._client._transport._credentials).__module__}.{type(self._client._transport._credentials).__qualname__}",
                    "credentialsInfo": getattr(self.transport._credentials, "get_cred_info", lambda: None)(),
                } if hasattr(self._client._transport, "_credentials") else {
                    "serviceName": "google.cloud.aiplatform.v1.SemanticGovernancePolicyService",
                    "credentialsType": None,
                }
            )

    async def create_semantic_governance_policy(self,
            request: Optional[Union[semantic_governance_policy_service.CreateSemanticGovernancePolicyRequest, dict]] = None,
            *,
            parent: Optional[str] = None,
            semantic_governance_policy: Optional[semantic_governance_policy_service.SemanticGovernancePolicy] = None,
            semantic_governance_policy_id: Optional[str] = None,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Union[float, object] = gapic_v1.method.DEFAULT,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
            ) -> operation_async.AsyncOperation:
        r"""Creates a SemanticGovernancePolicy.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from googlecloudsdk.generated_clients.gapic_clients import aiplatform_v1

            async def sample_create_semantic_governance_policy():
                # Create a client
                client = aiplatform_v1.SemanticGovernancePolicyServiceAsyncClient()

                # Initialize request argument(s)
                semantic_governance_policy = aiplatform_v1.SemanticGovernancePolicy()
                semantic_governance_policy.natural_language_constraint = "natural_language_constraint_value"
                semantic_governance_policy.agent = "agent_value"

                request = aiplatform_v1.CreateSemanticGovernancePolicyRequest(
                    parent="parent_value",
                    semantic_governance_policy=semantic_governance_policy,
                    semantic_governance_policy_id="semantic_governance_policy_id_value",
                )

                # Make the request
                operation = client.create_semantic_governance_policy(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.CreateSemanticGovernancePolicyRequest, dict]]):
                The request object. Request message for
                SemanticGovernancePolicyService.CreateSemanticGovernancePolicy.
            parent (:class:`str`):
                Required. The resource name of the Location into which
                to create the SemanticGovernancePolicy. Format:
                ``projects/{project}/locations/{location}``

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            semantic_governance_policy (:class:`googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.SemanticGovernancePolicy`):
                Required. The
                SemanticGovernancePolicy to create.

                This corresponds to the ``semantic_governance_policy`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            semantic_governance_policy_id (:class:`str`):
                Required. The ID to use for the
                SemanticGovernancePolicy, which will become the final
                component of the SemanticGovernancePolicy's resource
                name.

                This value may be up to 63 characters, and valid
                characters are ``[a-z0-9-]``. The first character cannot
                be a number or hyphen. The last character must be a
                letter or a number.

                This corresponds to the ``semantic_governance_policy_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.SemanticGovernancePolicy` Represents a governance policy applied to a specific Agent and optionally
                   a specific Tool within that Agent.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [parent, semantic_governance_policy, semantic_governance_policy_id]
        has_flattened_params = len([param for param in flattened_params if param is not None]) > 0
        if request is not None and has_flattened_params:
            raise ValueError("If the `request` argument is set, then none of "
                             "the individual field arguments should be set.")

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, semantic_governance_policy_service.CreateSemanticGovernancePolicyRequest):
            request = semantic_governance_policy_service.CreateSemanticGovernancePolicyRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent
        if semantic_governance_policy is not None:
            request.semantic_governance_policy = semantic_governance_policy
        if semantic_governance_policy_id is not None:
            request.semantic_governance_policy_id = semantic_governance_policy_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[self._client._transport.create_semantic_governance_policy]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((
                ("parent", request.parent),
            )),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            semantic_governance_policy_service.SemanticGovernancePolicy,
            metadata_type=semantic_governance_policy_service.CreateSemanticGovernancePolicyOperationMetadata,
        )

        # Done; return the response.
        return response

    async def get_semantic_governance_policy(self,
            request: Optional[Union[semantic_governance_policy_service.GetSemanticGovernancePolicyRequest, dict]] = None,
            *,
            name: Optional[str] = None,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Union[float, object] = gapic_v1.method.DEFAULT,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
            ) -> semantic_governance_policy_service.SemanticGovernancePolicy:
        r"""Gets a SemanticGovernancePolicy.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from googlecloudsdk.generated_clients.gapic_clients import aiplatform_v1

            async def sample_get_semantic_governance_policy():
                # Create a client
                client = aiplatform_v1.SemanticGovernancePolicyServiceAsyncClient()

                # Initialize request argument(s)
                request = aiplatform_v1.GetSemanticGovernancePolicyRequest(
                    name="name_value",
                )

                # Make the request
                response = await client.get_semantic_governance_policy(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.GetSemanticGovernancePolicyRequest, dict]]):
                The request object. Request message for
                SemanticGovernancePolicyService.GetSemanticGovernancePolicy.
            name (:class:`str`):
                Required. The name of the SemanticGovernancePolicy
                resource. Format:
                ``projects/{project}/locations/{location}/semanticGovernancePolicies/{semantic_governance_policy}``

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.SemanticGovernancePolicy:
                Represents a governance policy
                applied to a specific Agent and
                optionally a specific Tool within that
                Agent.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [name]
        has_flattened_params = len([param for param in flattened_params if param is not None]) > 0
        if request is not None and has_flattened_params:
            raise ValueError("If the `request` argument is set, then none of "
                             "the individual field arguments should be set.")

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, semantic_governance_policy_service.GetSemanticGovernancePolicyRequest):
            request = semantic_governance_policy_service.GetSemanticGovernancePolicyRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[self._client._transport.get_semantic_governance_policy]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((
                ("name", request.name),
            )),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_semantic_governance_policies(self,
            request: Optional[Union[semantic_governance_policy_service.ListSemanticGovernancePoliciesRequest, dict]] = None,
            *,
            parent: Optional[str] = None,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Union[float, object] = gapic_v1.method.DEFAULT,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
            ) -> pagers.ListSemanticGovernancePoliciesAsyncPager:
        r"""Lists SemanticGovernancePolicies in a given location.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from googlecloudsdk.generated_clients.gapic_clients import aiplatform_v1

            async def sample_list_semantic_governance_policies():
                # Create a client
                client = aiplatform_v1.SemanticGovernancePolicyServiceAsyncClient()

                # Initialize request argument(s)
                request = aiplatform_v1.ListSemanticGovernancePoliciesRequest(
                    parent="parent_value",
                )

                # Make the request
                page_result = client.list_semantic_governance_policies(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.ListSemanticGovernancePoliciesRequest, dict]]):
                The request object. Request message for
                SemanticGovernancePolicyService.ListSemanticGovernancePolicies.
            parent (:class:`str`):
                Required. The resource name of the Location from which
                to list the SemanticGovernancePolicies. Format:
                ``projects/{project}/locations/{location}``

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.services.semantic_governance_policy_service.pagers.ListSemanticGovernancePoliciesAsyncPager:
                Response message for
                SemanticGovernancePolicyService.ListSemanticGovernancePolicies.

                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [parent]
        has_flattened_params = len([param for param in flattened_params if param is not None]) > 0
        if request is not None and has_flattened_params:
            raise ValueError("If the `request` argument is set, then none of "
                             "the individual field arguments should be set.")

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, semantic_governance_policy_service.ListSemanticGovernancePoliciesRequest):
            request = semantic_governance_policy_service.ListSemanticGovernancePoliciesRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[self._client._transport.list_semantic_governance_policies]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((
                ("parent", request.parent),
            )),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListSemanticGovernancePoliciesAsyncPager(
            method=rpc,
            request=request,
            response=response,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def update_semantic_governance_policy(self,
            request: Optional[Union[semantic_governance_policy_service.UpdateSemanticGovernancePolicyRequest, dict]] = None,
            *,
            semantic_governance_policy: Optional[semantic_governance_policy_service.SemanticGovernancePolicy] = None,
            update_mask: Optional[field_mask_pb2.FieldMask] = None,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Union[float, object] = gapic_v1.method.DEFAULT,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
            ) -> operation_async.AsyncOperation:
        r"""Updates a SemanticGovernancePolicy.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from googlecloudsdk.generated_clients.gapic_clients import aiplatform_v1

            async def sample_update_semantic_governance_policy():
                # Create a client
                client = aiplatform_v1.SemanticGovernancePolicyServiceAsyncClient()

                # Initialize request argument(s)
                semantic_governance_policy = aiplatform_v1.SemanticGovernancePolicy()
                semantic_governance_policy.natural_language_constraint = "natural_language_constraint_value"
                semantic_governance_policy.agent = "agent_value"

                request = aiplatform_v1.UpdateSemanticGovernancePolicyRequest(
                    semantic_governance_policy=semantic_governance_policy,
                )

                # Make the request
                operation = client.update_semantic_governance_policy(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.UpdateSemanticGovernancePolicyRequest, dict]]):
                The request object. Request message for
                SemanticGovernancePolicyService.UpdateSemanticGovernancePolicy.
            semantic_governance_policy (:class:`googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.SemanticGovernancePolicy`):
                Required. The SemanticGovernancePolicy to update.

                The SemanticGovernancePolicy's ``name`` field is used to
                identify the SemanticGovernancePolicy to update. Format:
                ``projects/{project}/locations/{location}/semanticGovernancePolicies/{semantic_governance_policy}``

                This corresponds to the ``semantic_governance_policy`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            update_mask (:class:`google.protobuf.field_mask_pb2.FieldMask`):
                Optional. ``update_mask`` is used to specify the fields
                to be overwritten in the SemanticGovernancePolicy
                resource by the update. The fields specified in the
                ``update_mask`` are relative to the resource, not the
                full request. A field will be overwritten if it is in
                the mask. If the mask is not present, then all fields
                that are populated in the request message will be
                overwritten. Set the ``update_mask`` to ``*`` to
                override all fields.

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.SemanticGovernancePolicy` Represents a governance policy applied to a specific Agent and optionally
                   a specific Tool within that Agent.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [semantic_governance_policy, update_mask]
        has_flattened_params = len([param for param in flattened_params if param is not None]) > 0
        if request is not None and has_flattened_params:
            raise ValueError("If the `request` argument is set, then none of "
                             "the individual field arguments should be set.")

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, semantic_governance_policy_service.UpdateSemanticGovernancePolicyRequest):
            request = semantic_governance_policy_service.UpdateSemanticGovernancePolicyRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if semantic_governance_policy is not None:
            request.semantic_governance_policy = semantic_governance_policy
        if update_mask is not None:
            request.update_mask = update_mask

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[self._client._transport.update_semantic_governance_policy]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((
                ("semantic_governance_policy.name", request.semantic_governance_policy.name),
            )),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            semantic_governance_policy_service.SemanticGovernancePolicy,
            metadata_type=semantic_governance_policy_service.UpdateSemanticGovernancePolicyOperationMetadata,
        )

        # Done; return the response.
        return response

    async def delete_semantic_governance_policy(self,
            request: Optional[Union[semantic_governance_policy_service.DeleteSemanticGovernancePolicyRequest, dict]] = None,
            *,
            name: Optional[str] = None,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Union[float, object] = gapic_v1.method.DEFAULT,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
            ) -> operation_async.AsyncOperation:
        r"""Deletes a SemanticGovernancePolicy.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from googlecloudsdk.generated_clients.gapic_clients import aiplatform_v1

            async def sample_delete_semantic_governance_policy():
                # Create a client
                client = aiplatform_v1.SemanticGovernancePolicyServiceAsyncClient()

                # Initialize request argument(s)
                request = aiplatform_v1.DeleteSemanticGovernancePolicyRequest(
                    name="name_value",
                )

                # Make the request
                operation = client.delete_semantic_governance_policy(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.DeleteSemanticGovernancePolicyRequest, dict]]):
                The request object. Request message for
                SemanticGovernancePolicyService.DeleteSemanticGovernancePolicy.
            name (:class:`str`):
                Required. The name of the SemanticGovernancePolicy
                resource to be deleted. Format:
                ``projects/{project}/locations/{location}/semanticGovernancePolicies/{semantic_governance_policy}``

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.protobuf.empty_pb2.Empty` A generic empty message that you can re-use to avoid defining duplicated
                   empty messages in your APIs. A typical example is to
                   use it as the request or the response type of an API
                   method. For instance:

                      service Foo {
                         rpc Bar(google.protobuf.Empty) returns
                         (google.protobuf.Empty);

                      }

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [name]
        has_flattened_params = len([param for param in flattened_params if param is not None]) > 0
        if request is not None and has_flattened_params:
            raise ValueError("If the `request` argument is set, then none of "
                             "the individual field arguments should be set.")

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, semantic_governance_policy_service.DeleteSemanticGovernancePolicyRequest):
            request = semantic_governance_policy_service.DeleteSemanticGovernancePolicyRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[self._client._transport.delete_semantic_governance_policy]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((
                ("name", request.name),
            )),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            empty_pb2.Empty,
            metadata_type=semantic_governance_policy_service.DeleteSemanticGovernancePolicyOperationMetadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "SemanticGovernancePolicyServiceAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(gapic_version=package_version.__version__)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):   # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = cloudsdk.google.protobuf.__version__


__all__ = (
    "SemanticGovernancePolicyServiceAsyncClient",
)



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for the GCP Device Cloud (device-run) API."""

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base


class UnsupportedReleaseTrackError(Exception):
  """Raised when requesting API version for an unsupported release track."""


def ReleaseTrackToApiVersion(release_track):
  if release_track == base.ReleaseTrack.ALPHA:
    return 'v1alpha'
  else:
    raise UnsupportedReleaseTrackError(release_track)


class DeviceRunClient(object):
  """Base client wrapper for Device Run API."""

  def __init__(self, api_version='v1alpha'):
    self.client = apis.GetClientInstance('devicerun', api_version)
    self.messages = self.client.MESSAGES_MODULE
    self._service = None

  @property
  def service(self):
    return self._service


class SessionsClient(DeviceRunClient):
  """Client for the Sessions service under Device Run API."""

  def __init__(self, api_version='v1alpha'):
    super(SessionsClient, self).__init__(api_version)
    self._service = self.client.projects_locations_sessions

  def Get(self, session_ref):
    """Gets an automation session of Device Run."""
    request = self.messages.DevicerunProjectsLocationsSessionsGetRequest(
        name=session_ref.RelativeName()
    )
    return self._service.Get(request)

  def List(
      self,
      parent_ref,
      limit=None,
      page_size=100,
      filter_str=None,
      order_by=None,
      view=None,
  ):
    """Lists automation sessions of Device Run."""
    request = self.messages.DevicerunProjectsLocationsSessionsListRequest(
        parent=parent_ref.RelativeName(),
        filter=filter_str,
        orderBy=order_by,
        view=view,
    )
    return list_pager.YieldFromList(
        self._service,
        request,
        batch_size=page_size,
        limit=limit,
        field='sessions',
        batch_size_attribute='pageSize',
    )

  def Create(self, parent_ref, session_id=None, request_id=None, session=None):
    """Creates an automation session of Device Run."""
    request = self.messages.DevicerunProjectsLocationsSessionsCreateRequest(
        parent=parent_ref.RelativeName(),
        requestId=request_id,
        sessionId=session_id,
        session=session,
    )
    return self._service.Create(request)

  def Delete(self, session_ref, request_id=None):
    """Deletes an automation session of Device Run."""
    request = self.messages.DevicerunProjectsLocationsSessionsDeleteRequest(
        name=session_ref.RelativeName(),
        requestId=request_id,
    )
    return self._service.Delete(request)

  def Cancel(self, session_ref):
    """Starts asynchronous cancellation on an automation session of Device Run."""
    request = self.messages.DevicerunProjectsLocationsSessionsCancelRequest(
        name=session_ref.RelativeName()
    )
    return self._service.Cancel(request)


class OperationsClient(DeviceRunClient):
  """Client for the Operations service under Device Run API."""

  def __init__(self, api_version='v1alpha'):
    super(OperationsClient, self).__init__(api_version)
    self._service = self.client.projects_locations_operations

  def Get(self, operation_ref):
    """Gets a resource operation of Device Run."""
    request = self.messages.DevicerunProjectsLocationsOperationsGetRequest(
        name=operation_ref.RelativeName()
    )
    return self._service.Get(request)

  def List(self, parent_ref, limit=None, page_size=100, filter_str=None):
    """Lists resource operations of Device Run."""
    request = self.messages.DevicerunProjectsLocationsOperationsListRequest(
        name=parent_ref.RelativeName(),
        filter=filter_str,
    )
    return list_pager.YieldFromList(
        self._service,
        request,
        batch_size=page_size,
        limit=limit,
        field='operations',
        batch_size_attribute='pageSize',
    )

  def Cancel(self, operation_ref):
    """Starts asynchronous cancellation on a long-running operation."""
    request = self.messages.DevicerunProjectsLocationsOperationsCancelRequest(
        name=operation_ref.RelativeName()
    )
    return self._service.Cancel(request)

  def Delete(self, operation_ref):
    """Deletes a long-running operation."""
    request = self.messages.DevicerunProjectsLocationsOperationsDeleteRequest(
        name=operation_ref.RelativeName()
    )
    return self._service.Delete(request)


class LocationsClient(DeviceRunClient):
  """Client for the Locations service under Device Run API."""

  def __init__(self, api_version='v1alpha'):
    super(LocationsClient, self).__init__(api_version)
    self._service = self.client.projects_locations

  def Get(self, location_ref):
    """Gets information about a location."""
    request = self.messages.DevicerunProjectsLocationsGetRequest(
        name=location_ref.RelativeName()
    )
    return self._service.Get(request)

  def List(self, project_ref, limit=None, page_size=100, filter_str=None):
    """Lists information about the supported locations for this service."""
    request = self.messages.DevicerunProjectsLocationsListRequest(
        name=project_ref.RelativeName(),
        filter=filter_str,
    )
    return list_pager.YieldFromList(
        self._service,
        request,
        batch_size=page_size,
        limit=limit,
        field='locations',
        batch_size_attribute='pageSize',
    )


class DeviceRunOperationPoller(waiter.OperationPoller):
  """Implementation of OperationPoller for Device Run LRO Operations."""

  def __init__(self, resource_service, operations_service, resource_ref=None):
    self.resource_service = resource_service
    self.operations_service = operations_service
    self.resource_ref = resource_ref

  def IsDone(self, operation):
    if operation.done:
      if operation.error:
        raise waiter.OperationError(operation.error.message)
      return True
    return False

  def Poll(self, operation_ref):
    request_type = self.operations_service.GetRequestType('Get')
    return self.operations_service.Get(
        request_type(name=operation_ref.RelativeName())
    )

  def GetResult(self, operation):
    if self.resource_ref is None:
      return operation
    request_type = self.resource_service.GetRequestType('Get')
    return self.resource_service.Get(
        request_type(name=self.resource_ref.RelativeName())
    )



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags for the compute instance groups managed commands."""


import collections
import sys

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.calliope import parser_arguments
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.util.apis import arg_utils


MULTI_MIG_ARG = compute_flags.ResourceArgument(
    '--multi-mig',
    resource_name='multi-MIG',
    required=False,
    plural=False,
    scope_flags_usage=compute_flags.ScopeFlagsUsage.USE_EXISTING_SCOPE_FLAGS,
    regional_collection='compute.regionMultiMigs',
    region_help_text="""
    Specifies the name or URL of the multi-MIG that will be attached to selected
    managed instance group.
    """,
    region_hidden=True,
)

DEFAULT_CREATE_OR_LIST_FORMAT = """\
    table(
      name,
      location():label=LOCATION,
      location_scope():label=SCOPE,
      baseInstanceName,
      size,
      targetSize,
      instanceTemplate.basename(),
      autoscaled
    )
"""


def AddTypeArg(parser):
  parser.add_argument(
      '--type',
      choices={
          'opportunistic': (
              'Do not proactively replace VMs. Create new VMs and delete old '
              'ones on resizes of the group and when you target specific VMs '
              'to be updated or recreated.'
          ),
          'proactive': 'Replace instances proactively.',
      },
      default='proactive',
      category=base.COMMONLY_USED_FLAGS,
      help='Desired update type.',
  )


def AddMaxSurgeArg(parser):
  parser.add_argument(
      '--max-surge',
      type=str,
      help=(
          'Maximum additional number of instances that '
          'can be created during the update process. '
          'This can be a fixed number (e.g. 5) or '
          'a percentage of size to the managed instance '
          'group (e.g. 10%). Defaults to 0 if the managed '
          'instance group has stateful configuration, or to '
          'the number of zones in which it operates otherwise.'
      ),
  )


def AddMaxUnavailableArg(parser):
  parser.add_argument(
      '--max-unavailable',
      type=str,
      help=(
          'Maximum number of instances that can be '
          'unavailable during the update process. '
          'This can be a fixed number (e.g. 5) or '
          'a percentage of size to the managed instance '
          'group (e.g. 10%). Defaults to the number of zones '
          'in which the managed instance group operates.'
      ),
  )


def AddMinReadyArg(parser):
  parser.add_argument(
      '--min-ready',
      type=arg_parsers.Duration(lower_bound='0s'),
      help=(
          'Minimum time for which a newly created instance '
          'should be ready to be considered available. For example `10s` '
          'for 10 seconds. See $ gcloud topic datetimes for information '
          'on duration formats.'
      ),
  )


def AddReplacementMethodFlag(parser):
  parser.add_argument(
      '--replacement-method',
      choices={
          'substitute': (
              'Delete old instances and create instances with new names.'
          ),
          'recreate': (
              'Recreate instances and preserve the instance names. '
              'The instance IDs and creation timestamps might change.'
          ),
      },
      help=(
          'Type of replacement method. Specifies what action will be taken to'
          ' update instances. Defaults to ``recreate`` if the managed instance'
          ' group has stateful configuration, or to ``substitute`` otherwise.'
      ),
  )


def AddForceArg(parser):
  parser.add_argument(
      '--force',
      action='store_true',
      help=(
          'If set, accepts any original or new version '
          'configurations without validation.'
      ),
  )


def InstanceTemplateArg(
    required=True,
    help_override=None,
) -> compute_flags.ResourceArgument:
  """Create a resource argument for instance templates.

  Args:
    required: Whether the argument is required.
    help_override: Optional help text to override the default.

  Returns:
    A compute_flags.ResourceArgument for instance templates.
  """
  if help_override is None:
    help_text = """
      Specifies the instance template to use when creating new instances.
      An instance template is either a global or regional resource.
      """
  else:
    help_text = help_override

  return compute_flags.ResourceArgument(
      '--template',
      resource_name='instance template',
      required=required,
      plural=False,
      scope_flags_usage=compute_flags.ScopeFlagsUsage.DONT_USE_SCOPE_FLAGS,
      global_collection='compute.instanceTemplates',
      regional_collection='compute.regionInstanceTemplates',
      short_help=help_text,
  )


def InstanceActionChoicesWithoutNone(flag_prefix=''):
  """Return possible instance action choices without NONE value."""
  return collections.OrderedDict([
      (
          'refresh',
          (
              'Apply the new configuration without stopping VMs, if possible.'
              ' For example, use ``refresh`` to apply changes that only affect'
              ' metadata or additional disks.'
          ),
      ),
      (
          'restart',
          (
              'Apply the new configuration without replacing VMs, if possible.'
              ' For example, stopping VMs and starting them again is sufficient'
              ' to apply changes to machine type.'
          ),
      ),
      (
          'replace',
          (
              'Replace old VMs according to the '
              '--{flag_prefix}replacement-method flag.'
          ).format(flag_prefix=flag_prefix),
      ),
  ])


def InstanceActionChoicesWithNone(flag_prefix=''):
  """Return possible instance action choices with NONE value."""
  return _CombineOrderedChoices(
      {'none': 'No action'}, InstanceActionChoicesWithoutNone(flag_prefix)
  )


def _CombineOrderedChoices(choices1, choices2):
  merged = collections.OrderedDict([])
  merged.update(choices1.items())
  merged.update(choices2.items())
  return merged


def AddMinimalActionArg(parser, choices_with_none=True, default=None):
  choices = (
      InstanceActionChoicesWithNone()
      if choices_with_none
      else InstanceActionChoicesWithoutNone()
  )
  parser.add_argument(
      '--minimal-action',
      choices=choices,
      default=default,
      help="""Use this flag to minimize disruption as much as possible or to
        apply a more disruptive action than is strictly necessary.
        The MIG performs at least this action on each instance while
        updating. If the update requires a more disruptive action than
        the one specified here, then the more disruptive action is
        performed. If you omit this flag, the update uses the
        ``minimal-action'' value from the MIG\'s update policy, unless it
        is not set in which case the default is ``replace''.""",
  )


def AddMostDisruptiveActionArg(parser, choices_with_none=True, default=None):
  choices = (
      InstanceActionChoicesWithNone()
      if choices_with_none
      else InstanceActionChoicesWithoutNone()
  )
  parser.add_argument(
      '--most-disruptive-allowed-action',
      choices=choices,
      default=default,
      help="""Use this flag to prevent an update if it requires more disruption
        than you can afford. At most, the MIG performs the specified
        action on each instance while updating. If the update requires
        a more disruptive action than the one specified here, then
        the update fails and no changes are made. If you omit this flag,
        the update uses the ``most-disruptive-allowed-action'' value from
        the MIG\'s update policy, unless it is not set in which case
        the default is ``replace''.""",
  )


def AddUpdateInstancesArgs(parser):
  """Add args for the update-instances command."""
  instance_selector_group = parser.add_group(required=True, mutex=True)
  instance_selector_group.add_argument(
      '--instances',
      type=arg_parsers.ArgList(min_length=1),
      metavar='INSTANCE',
      required=False,
      help='Names of instances to update.',
  )
  instance_selector_group.add_argument(
      '--all-instances',
      required=False,
      action='store_true',
      help='Update all instances in the group.',
  )
  AddMinimalActionArg(parser, True, 'none')
  AddMostDisruptiveActionArg(parser, True, 'replace')


def AddGracefulValidationArg(parser):
  help_text = """Specifies whether the request should proceed even if the
    request includes instances that are not members of the group or that are
    already being deleted or abandoned. By default, if you omit this flag and
    such an instance is specified in the request, the operation fails. The
    operation always fails if the request contains a badly formatted instance
    name or a reference to an instance that exists in a zone or region other
    than the group's zone or region."""
  parser.add_argument(
      '--skip-instances-on-validation-error',
      action='store_true',
      help=help_text,
  )


def GetCommonPerInstanceCommandOutputFormat(with_validation_error=False):
  if with_validation_error:
    return """
        table(project(),
              zone(),
              instanceName:label=INSTANCE,
              status,
              validationError:label=VALIDATION_ERROR)"""
  else:
    return """
        table(project(),
              zone(),
              instanceName:label=INSTANCE,
              status)"""


INSTANCE_REDISTRIBUTION_TYPES = ['NONE', 'PROACTIVE']


def AddMigUpdatePolicyFlags(parser, support_min_ready_flag=False):
  """Add flags required for setting update policy attributes."""
  group = parser.add_group(
      required=False,
      mutex=False,
      help=(
          'Parameters for setting update policy for this managed instance '
          'group.'
      ),
  )
  _AddUpdatePolicyTypeFlag(group)
  _AddUpdatePolicyMaxUnavailableFlag(group)
  _AddUpdatePolicyMaxSurgeFlag(group)
  _AddUpdatePolicyMinimalActionFlag(group)
  _AddUpdatePolicyMostDisruptiveActionFlag(group)
  _AddUpdatePolicyReplacementMethodFlag(group)
  if support_min_ready_flag:
    _AddUpdatePolicyMinReadyFlag(group)


def _AddUpdatePolicyTypeFlag(group):
  """Add --update-policy-type flag to the parser."""
  help_text = (
      'Specifies the type of update process. You can specify either '
      '``proactive`` so that the managed instance group proactively '
      'executes actions in order to bring VMs to their target '
      'versions or ``opportunistic`` so that no action is '
      'proactively executed but the update will be performed as part '
      'of other actions.'
  )
  choices = {
      'opportunistic': (
          'Do not proactively replace VMs. Create new VMs and delete old ones '
          'on resizes of the group and when you target specific VMs to be '
          'updated or recreated.'
      ),
      'proactive': 'Replace VMs proactively.',
  }
  group.add_argument(
      '--update-policy-type',
      metavar='UPDATE_TYPE',
      type=lambda x: x.lower(),
      choices=choices,
      help=help_text,
  )


def _AddUpdatePolicyMaxUnavailableFlag(group):
  group.add_argument(
      '--update-policy-max-unavailable',
      metavar='MAX_UNAVAILABLE',
      type=str,
      help=(
          'Maximum number of VMs that can be unavailable during the update '
          'process. This can be a fixed number (e.g. 5) or a percentage of '
          'size to the managed instance group (e.g. 10%). Defaults to the '
          'number of zones in which the managed instance group operates.'
      ),
  )


def _AddUpdatePolicyMaxSurgeFlag(group):
  group.add_argument(
      '--update-policy-max-surge',
      metavar='MAX_SURGE',
      type=str,
      help=(
          'Maximum additional number of VMs that can be created during the '
          'update process. This can be a fixed number (e.g. 5) or a percentage '
          'of size to the managed instance group (e.g. 10%).'
      ),
  )


def _AddUpdatePolicyMinReadyFlag(group):
  group.add_argument(
      '--update-policy-min-ready',
      metavar='MIN_READY',
      type=arg_parsers.Duration(lower_bound='0s'),
      help=(
          'Minimum time for which a newly created VM should be ready to be '
          'considered available. For example `10s` for 10 seconds. See '
          '$ gcloud topic datetimes for information on duration formats.'
      ),
  )


def _AddUpdatePolicyMinimalActionFlag(group):
  group.add_argument(
      '--update-policy-minimal-action',
      choices=InstanceActionChoicesWithNone(flag_prefix='update-policy-'),
      help=(
          'Use this flag to minimize disruption as much as possible or to '
          'apply a more disruptive action than is strictly necessary. '
          'The MIG performs at least this action on each VM while '
          'updating. If the update requires a more disruptive action than '
          'the one specified here, then the more disruptive action is '
          'performed. '
      ),
  )


def _AddUpdatePolicyMostDisruptiveActionFlag(group):
  group.add_argument(
      '--update-policy-most-disruptive-action',
      choices=InstanceActionChoicesWithNone(flag_prefix='update-policy-'),
      help=(
          'Use this flag to prevent an update if it requires more disruption '
          'than you can afford. At most, the MIG performs the specified '
          'action on each VM while updating. If the update requires '
          'a more disruptive action than the one specified here, then '
          'the update fails and no changes are made.'
      ),
  )


def _AddUpdatePolicyReplacementMethodFlag(group):
  group.add_argument(
      '--update-policy-replacement-method',
      choices={
          'substitute': 'Delete old VMs and create VMs with new names.',
          'recreate': (
              'Recreate VMs and preserve the VM names. '
              'The VM IDs and creation timestamps might change.'
          ),
      },
      help=(
          'Type of replacement method. Specifies what action will be taken '
          'to update VMs.'
      ),
  )


def AddMigInstanceRedistributionTypeFlag(parser):
  """Add --instance-redistribution-type flag to the parser."""
  help_text = """\
      Specifies the type of the instance redistribution policy. An instance
      redistribution type lets you enable or disable automatic instance
      redistribution across zones to meet the group's target distribution shape.

      An instance redistribution type can be specified only for a non-autoscaled
      regional managed instance group. By default it is set to ``proactive''.
      """
  choices = {
      'none': (
          'The managed instance group does not redistribute instances across '
          'zones.'
      ),
      'proactive': (
          'The managed instance group proactively redistributes instances to '
          'meet its target distribution.'
      ),
  }

  parser.add_argument(
      '--instance-redistribution-type',
      metavar='TYPE',
      type=lambda x: x.lower(),
      choices=choices,
      help=help_text,
  )


def AddMigDistributionPolicyTargetShapeFlag(parser):
  """Add --target-distribution-shape flag to the parser."""
  help_text = """\
      Specifies how a regional managed instance group distributes its instances
      across zones within the region. The default shape is ``even''.
    """
  choices = {
      'even': (
          'The group schedules VM instance creation and deletion to achieve '
          'and maintain an even number of managed instances across the '
          'selected zones. The distribution is even when the number of managed'
          ' instances does not differ by more than 1 between any two zones. '
          'Recommended for highly available serving workloads.'
      ),
      'balanced': (
          'The group prioritizes acquisition of resources, scheduling VMs in '
          'zones where resources are available while distributing VMs as '
          'evenly as possible across selected zones to minimize the impact of '
          'zonal failure. Recommended for highly available serving or batch '
          'workloads that do not require autoscaling.'
      ),
      'any': (
          'The group picks zones for creating VM instances to fulfill the '
          'requested number of VMs within present resource constraints and to '
          'maximize utilization of unused zonal reservations. Recommended for '
          'batch workloads that do not require high availability.'
      ),
      'any-single-zone': (
          'The group schedules all instances within a single zone. The zone '
          'is chosen based on hardware support, current resources '
          'availability, and matching reservations. The group might not be '
          'able to create the requested number of VMs in case of zonal '
          'resource availability constraints. Recommended for workloads '
          'requiring extensive communication between VMs.'
      ),
  }

  parser.add_argument(
      '--target-distribution-shape',
      metavar='SHAPE',
      type=arg_utils.EnumNameToChoice,
      choices=choices,
      help=help_text,
  )


def AddFlagsForUpdateAllInstancesConfig(parser):
  """Adds args for all-instances' config update command."""
  # Add  metadata args
  metadata_argument_name = '--metadata'
  metadata_help_text = (
      "Add metadata to the group's all instances configuration."
  )
  parser.add_argument(
      metadata_argument_name,
      type=arg_parsers.ArgDict(min_length=1),
      default={},
      action=arg_parsers.StoreOnceAction,
      metavar='KEY=VALUE',
      help=metadata_help_text,
  )
  # Add labels args
  labels_argument_name = '--labels'
  metadata_help_text = "Add labels to the group's all instances configuration."
  parser.add_argument(
      labels_argument_name,
      type=arg_parsers.ArgDict(min_length=1),
      default={},
      action=arg_parsers.StoreOnceAction,
      metavar='KEY=VALUE',
      help=metadata_help_text,
  )


def AddFlagsForDeleteAllInstancesConfig(parser):
  """Adds args for all-instances' config delete command."""
  # Add  metadata args
  metadata_argument_name = '--metadata'
  parser.add_argument(
      metadata_argument_name,
      metavar='KEY',
      type=arg_parsers.ArgList(min_length=1),
      help="Remove metadata keys from the group's all instances configuration.",
  )
  # Add labels args
  labels_argument_name = '--labels'
  parser.add_argument(
      labels_argument_name,
      metavar='KEY',
      type=arg_parsers.ArgList(min_length=1),
      help="Remove labels keys from the group's all instances configuration.",
  )


def ValidateRegionalMigFlagsUsage(args, regional_flags_dests, igm_ref):
  """For zonal MIGs validate that user did not supply any RMIG-specific flags.

  Can be safely called from GA track for all flags, unknowns are ignored.

  Args:
    args: provided arguments.
    regional_flags_dests: list of RMIG-specific flag dests (names of the
      attributes used to store flag values in args).
    igm_ref: resource reference of the target IGM.
  """
  if igm_ref.Collection() == 'compute.regionInstanceGroupManagers':
    return
  for dest in regional_flags_dests:
    if args.IsKnownAndSpecified(dest):
      flag_name = args.GetFlag(dest)
      error_message = (
          'Flag %s may be specified for regional managed instance groups only.'
      ) % flag_name
      raise exceptions.InvalidArgumentException(
          parameter_name=flag_name, message=error_message
      )


def AddMigListManagedInstancesResultsFlag(parser):
  """Add --list-managed-instances-results flag to the parser."""
  help_text = """\
      Pagination behavior for the group's listManagedInstances API method.
      This flag does not affect the group's gcloud or console list-instances
      behavior. By default it is set to ``pageless''.
    """
  choices = {
      'pageless': (
          "Pagination is disabled for the group's listManagedInstances API "
          'method. maxResults and pageToken query parameters are ignored and '
          'all instances are returned in a single response.'
      ),
      'paginated': (
          "Pagination is enabled for the group's listManagedInstances API "
          'method. maxResults and pageToken query parameters are respected.'
      ),
  }

  parser.add_argument(
      '--list-managed-instances-results',
      metavar='MODE',
      type=lambda x: x.lower(),
      choices=choices,
      help=help_text,
  )


def AddMigForceUpdateOnRepairFlags(parser):
  """Adding force update on repair flag to the parser."""
  help_text = """
      Specifies whether to apply the group's latest configuration when
      repairing a VM. If you updated the group's instance template or
      per-instance configurations after the VM was created, then these changes
      are applied when VM is repaired. If this flag is disabled with
      ``-no-force-update-on-repair'', then updates are applied in accordance
      with the group's update policy type. By default, this flag is disabled.
    """
  parser.add_argument(
      '--force-update-on-repair',
      action=arg_parsers.StoreTrueFalseAction,
      help=help_text,
  )


def AddMigDefaultActionOnVmFailure(parser, release_track):
  if release_track == base.ReleaseTrack.ALPHA:
    AddMigDefaultActionOnVmFailureAlpha(parser)
  else:
    # For GA and BETA tracks, the default action on VM failure is the same.
    AddMigDefaultActionOnVmFailureGA(parser)


def AddMigDefaultActionOnVmFailureGA(parser):
  """Add default action on VM failure to the parser."""
  help_text = """\
      Specifies the action that a MIG performs on a failed VM. If the value of
      the onFailedHealthCheck field is `DEFAULT_ACTION`, then the same action
      also applies to the VMs on which your application fails a health check.
      By default, the value of the flag is set to ``repair''."""
  choices = collections.OrderedDict([
      (
          'repair',
          '(Default) MIG automatically repairs a failed VM by recreating it.',
      ),
      ('do-nothing', 'MIG does not repair a failed VM.'),
  ])

  parser.add_argument(
      '--default-action-on-vm-failure',
      metavar='ACTION_ON_VM_FAILURE',
      type=arg_utils.EnumNameToChoice,
      choices=choices,
      help=help_text,
  )


def AddMigDefaultActionOnVmFailureAlpha(parser):
  """Add default action on VM failure to the parser."""
  help_text = """\
      Specifies the action that a MIG performs on a failed VM. If the value of
      the onFailedHealthCheck field is `DEFAULT_ACTION`, then the same action
      also applies to the VMs on which your application fails a health check.
      By default, the value of the flag is set to ``repair''."""
  choices = collections.OrderedDict([
      (
          'repair',
          '(Default) MIG automatically repairs a failed VM by recreating it.',
      ),
      ('do-nothing', 'MIG does not repair a failed VM.'),
      ('delete', ('MIG deletes a failed VM. Deleting the VM decreases the '
                  'target size of the MIG.')),
  ])

  parser.add_argument(
      '--default-action-on-vm-failure',
      metavar='ACTION_ON_VM_FAILURE',
      type=arg_utils.EnumNameToChoice,
      choices=choices,
      help=help_text,
  )


def AddMigActionOnVmFailedHealthCheck(parser):
  """Add action on VM failed health check to the parser."""
  help_text = """\
      Specifies the action that a MIG performs on an unhealthy VM.
      A VM is marked as unhealthy when the application running on that VM fails
      a health check. By default, the value of the flag is set to ``default-action''."""
  choices = collections.OrderedDict([
      (
          'default-action',
          (
              '(Default) MIG uses the same action configured for the '
              'defaultActionOnFailure field.'
          ),
      ),
      ('do-nothing', 'MIG does not repair an unhealthy VM.'),
      ('repair', 'MIG automatically repairs an unhealthy VM by recreating it.'),
  ])

  parser.add_argument(
      '--action-on-vm-failed-health-check',
      metavar='ACTION_ON_FAILED_HEALTH_CHECK',
      type=arg_utils.EnumNameToChoice,
      choices=choices,
      help=help_text,
  )


def AddMigSizeFlag(parser, required=False):
  """Add --size flag to the parser."""
  parser.add_argument(
      '--size',
      required=required,
      type=arg_parsers.BoundedInt(0, sys.maxsize, unlimited=True),
      help='Target number of running instances in managed instance group.',
  )


def AddStandbyPolicyFlags(parser):
  """Add flags required for setting standby policy."""
  standby_policy_mode_choices = {
      'manual': (
          'MIG does not automatically resume or start VMs in the standby pool'
          ' when the group scales out.'
      ),
      'scale-out-pool': (
          'MIG automatically resumes or starts VMs in the standby pool when the'
          ' group scales out, and replenishes the standby pool afterwards.'
      ),
  }
  parser.add_argument(
      '--standby-policy-mode',
      type=str,
      choices=standby_policy_mode_choices,
      help=("""\
          Defines how a MIG resumes or starts VMs from a standby pool when the\
          group scales out. The default mode is ``manual''.
      """),
  )
  parser.add_argument(
      '--standby-policy-initial-delay',
      type=int,
      help=(
          'Specifies the number of seconds that the MIG should wait before'
          ' suspending or stopping a VM. The initial delay gives the'
          ' initialization script the time to prepare your VM for a quick scale'
          ' out.'
      ),
  )
  parser.add_argument(
      '--suspended-size',
      type=int,
      help='Specifies the target size of suspended VMs in the group.',
  )
  parser.add_argument(
      '--stopped-size',
      type=int,
      help='Specifies the target size of stopped VMs in the group.',
  )


def AddMigResourceManagerTagsFlags(parser):
  """Adds resource manager tag related flags."""
  parser.add_argument(
      '--resource-manager-tags',
      type=arg_parsers.ArgDict(),
      metavar='KEY=VALUE',
      action=arg_parsers.UpdateAction,
      help=(
          'Specifies a list of resource manager tags to apply to the managed'
          ' instance group. A resource manager tag is a key-value pair. You'
          ' can attach exactly one value to a MIG for a given key. A MIG can'
          ' have a maximum of 50 key-value pairs attached.'
      ),
  )


def AddTargetSizePolicyModeFlag(parser):
  """Add target size policy mode to the parser."""
  help_text = 'Specifies the mode in which the MIG creates VMs in the group.'
  choices = {
      'individual': (
          '(Default) MIG creates VMs individually. If the MIG cannot create'
          ' all VMs to meet the specified ``size``, then it creates VMs for'
          ' which resources are available and continues to attempt to create'
          ' the remaining ones until the target size is met.'
      ),
      'bulk': (
          'MIG creates VMs all at once. If the MIG cannot create any VM to meet'
          ' the specified ``size``, then the MIG waits until the resources'
          ' become available to create all VMs.'
      ),
  }

  parser.add_argument(
      '--target-size-policy-mode',
      metavar='TARGET_SIZE_POLICY_MODE',
      type=arg_utils.EnumNameToChoice,
      choices=choices,
      help=help_text,
  )


def AddWorkloadPolicyFlag(parser: parser_arguments.ArgumentInterceptor):
  """Add workload policy flag."""
  parser.add_argument(
      '--workload-policy',
      type=str,
      metavar='WORKLOAD_POLICY',
      help=(
          'Specifies the workload policy for the managed instance group. It '
          'can be a full or partial URL to a resource policy containing '
          'the workload policy.'
      ),
  )


def AddRemoveWorkloadPolicyFlag(parser: parser_arguments.ArgumentInterceptor):
  """Add remove workload policy flag."""
  parser.add_argument(
      '--remove-workload-policy',
      action='store_true',
      help=(
          'Detaches the workload policy from the managed instance group.'
      ),
  )


def AddWorkloadPolicyFlags(parser: parser_arguments.ArgumentInterceptor):
  """Add flags for managing workload policy."""
  workload_policy_group = parser.add_group(mutex=True)
  AddWorkloadPolicyFlag(workload_policy_group)
  AddRemoveWorkloadPolicyFlag(workload_policy_group)


def AddOnRepairFlags(parser: parser_arguments.ArgumentInterceptor):
  """Add on repair flags to the parser."""
  allow_changing_zone_choices = {
      'no': "(Default) MIG cannot change a VM's zone during a repair.",
      'yes': 'MIG can select a different zone for the VM during a repair.',
  }
  parser.add_argument(
      '--on-repair-allow-changing-zone',
      metavar='ON_REPAIR_ALLOW_CHANGING_ZONE',
      type=arg_utils.EnumNameToChoice,
      choices=allow_changing_zone_choices,
      help="Specifies whether the MIG can change a VM's zone during a repair.",
  )


def AddMultiMigFlags(parser):
  """Adds Multi-MIG flags to the parser."""
  multi_mig_group = parser.add_mutually_exclusive_group(hidden=True)
  multi_mig_group.add_argument(
      '--clear-multi-mig',
      action='store_true',
      default=None,
      help="""\
      Removes this managed instance group from the Multi-MIG it belongs to.
      """,
  )
  MULTI_MIG_ARG.AddArgument(multi_mig_group)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update endpoint command."""


import datetime
from googlecloudsdk.api_lib.network_security.firewall_endpoints import activation_api
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.network_security import activation_flags
from googlecloudsdk.command_lib.util.args import labels_util

DETAILED_HELP = {
    'DESCRIPTION': """
          Update a firewall endpoint. Check the progress of endpoint update
          by using `gcloud network-security firewall-endpoints describe`.

          For more examples, refer to the EXAMPLES section below.

        """,
    'EXAMPLES': """
            To update labels k1 and k2, run:

            $ {command} my-endpoint --zone=us-central1-a --organization=1234 --update-labels=k1=v1,k2=v2

            To remove labels k3 and k4, run:

            $ {command} my-endpoint --zone=us-central1-a --organization=1234 --remove-labels=k3,k4

            To clear all labels from the firewall endpoint, run:

            $ {command} my-endpoint --zone=us-central1-a --organization=1234 --clear-labels
        """,
}

_PROJECT_SCOPE_SUPPORTED_TRACKS = (
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
)


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class Update(base.UpdateCommand):
  """Update a Firewall Plus endpoint.

  This command is used to update labels on the endpoint.
  """

  @classmethod
  def Args(cls, parser):
    project_scope_supported = (
        cls.ReleaseTrack() in _PROJECT_SCOPE_SUPPORTED_TRACKS
    )
    activation_flags.AddEndpointResource(
        cls.ReleaseTrack(),
        parser,
        project_scope_supported,
    )
    activation_flags.AddLocationArg(parser)
    activation_flags.AddMaxWait(parser, '60m')  # default to 60 minutes wait.
    activation_flags.AddDescriptionArg(parser)
    activation_flags.AddUpdateBillingProjectArg(parser)
    base.ASYNC_FLAG.AddToParser(parser)
    base.ASYNC_FLAG.SetDefault(parser, True)
    labels_util.AddUpdateLabelsFlags(parser)
    if cls.ReleaseTrack() in (base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA):
      activation_flags.AddEnableWildfireArg(parser)
      activation_flags.AddWildfireRegionArg(parser)
      activation_flags.AddContentCloudRegionArg(parser)
      activation_flags.AddWildfireLookupTimeoutArg(parser)
      activation_flags.AddWildfireLookupActionArg(parser)
      activation_flags.AddWildfireAnalysisTimeoutArg(parser)
      activation_flags.AddWildfireAnalysisActionArg(parser)
      activation_flags.AddEnableWildfireAnalysisLoggingArg(parser)
      activation_flags.AddBlockPartialHttpArg(parser)

  def Run(self, args):
    result = args.CONCEPTS.firewall_endpoint.Parse()
    endpoint = result.result

    project_scoped = (
        result.concept_type.name
        == activation_flags.PROJECT_ENDPOINT_RESOURCE_COLLECTION
    )

    client = activation_api.Client(self.ReleaseTrack(), project_scoped)

    original = client.DescribeEndpoint(endpoint.RelativeName())
    if original is None:
      raise exceptions.InvalidArgumentException(
          'firewall-endpoint',
          'Firewall endpoint does not exist.')

    update_mask = []

    labels = None
    labels_diff = labels_util.Diff.FromUpdateArgs(args)
    if labels_diff.MayHaveUpdates():
      update_mask.append('labels')
      labels = original.labels
      labels_update = labels_diff.Apply(
          client.messages.FirewallEndpoint.LabelsValue,
          original.labels,
      )
      if labels_update.needs_update:
        labels = labels_update.labels

    billing_project_id = args.update_billing_project
    if billing_project_id:
      update_mask.append('billing_project_id')

    if hasattr(args, 'enable_wildfire') and args.IsSpecified('enable_wildfire'):
      update_mask.append('wildfire_settings.enabled')
    if hasattr(args, 'wildfire_region') and args.IsSpecified('wildfire_region'):
      update_mask.append('wildfire_settings.wildfire_region')
    if hasattr(args, 'content_cloud_region') and args.IsSpecified(
        'content_cloud_region'
    ):
      update_mask.append('endpoint_settings.content_cloud_region')
    if hasattr(args, 'wildfire_lookup_timeout') and args.IsSpecified(
        'wildfire_lookup_timeout'
    ):
      update_mask.append('wildfire_settings.wildfire_realtime_lookup_duration')
    if hasattr(args, 'wildfire_lookup_action') and args.IsSpecified(
        'wildfire_lookup_action'
    ):
      update_mask.append(
          'wildfire_settings.wildfire_realtime_lookup_timeout_action'
      )
    if hasattr(args, 'wildfire_analysis_timeout') and args.IsSpecified(
        'wildfire_analysis_timeout'
    ):
      update_mask.append(
          'wildfire_settings.wildfire_inline_cloud_analysis_settings.max_analysis_duration'
      )
    if hasattr(args, 'wildfire_analysis_action') and args.IsSpecified(
        'wildfire_analysis_action'
    ):
      update_mask.append(
          'wildfire_settings.wildfire_inline_cloud_analysis_settings.timeout_action'
      )
    if hasattr(
        args, 'enable_wildfire_analysis_logging'
    ) and args.IsSpecified('enable_wildfire_analysis_logging'):
      update_mask.append(
          'wildfire_settings.wildfire_inline_cloud_analysis_settings.submission_timeout_logging_disabled'
      )
    if hasattr(args, 'block_partial_http') and args.IsSpecified(
        'block_partial_http'
    ):
      update_mask.append('endpoint_settings.http_partial_response_blocked')

    if not update_mask:
      if self.ReleaseTrack() == base.ReleaseTrack.ALPHA:
        raise exceptions.MinimumArgumentException([
            '--clear-labels',
            '--remove-labels',
            '--update-labels',
            '--update-billing-project',
            '--enable-wildfire',
            '--wildfire-region',
            '--content-cloud-region',
            '--wildfire-lookup-timeout',
            '--wildfire-lookup-action',
            '--wildfire-analysis-timeout',
            '--wildfire-analysis-action',
            '--enable-wildfire-analysis-logging',
            '--block-partial-http',
        ])
      else:
        raise exceptions.MinimumArgumentException([
            '--clear-labels',
            '--remove-labels',
            '--update-labels',
            '--update-billing-project',
        ])

    is_async = args.async_
    max_wait = datetime.timedelta(seconds=args.max_wait)

    operation = client.UpdateEndpoint(
        name=endpoint.RelativeName(),
        description=getattr(args, 'description', None),
        update_mask=','.join(update_mask),
        labels=labels,
        billing_project_id=billing_project_id,
        enable_wildfire=getattr(args, 'enable_wildfire', None),
        wildfire_region=getattr(args, 'wildfire_region', None),
        content_cloud_region=getattr(args, 'content_cloud_region', None),
        wildfire_lookup_timeout=getattr(args, 'wildfire_lookup_timeout', None),
        wildfire_lookup_action=getattr(args, 'wildfire_lookup_action', None),
        wildfire_analysis_timeout=getattr(
            args, 'wildfire_analysis_timeout', None
        ),
        wildfire_analysis_action=getattr(
            args, 'wildfire_analysis_action', None
        ),
        enable_wildfire_analysis_logging=getattr(
            args, 'enable_wildfire_analysis_logging', None
        ),
        block_partial_http=getattr(args, 'block_partial_http', None),
    )
    # Return the in-progress operation if async is requested.
    if is_async:
      # Delete operations have no format by default,
      # but here we want the operation metadata to be printed.
      if not args.IsSpecified('format'):
        args.format = 'default'
      return operation
    return client.WaitForOperation(
        operation_ref=client.GetOperationRef(operation),
        message='waiting for firewall endpoint [{}] to be updated'.format(
            endpoint.RelativeName()
        ),
        has_result=True,
        max_wait=max_wait,
    )


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.DefaultUniverseOnly
class UpdateAlpha(Update):
  """Update a Firewall Plus endpoint."""


Update.detailed_help = DETAILED_HELP
